open Pretty
open Jabs

let rec d_ident () (a1 : ident) =
  dprintf "\"%s\"" a1

let rec d_qualified_ident () (a2 : qualified_ident) =
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_ident () elt)) a2


let rec d_compilation_unit () (a3 : compilation_unit) =
  text "{ " ++ align ++ text "cu_filename = " ++ break ++
  dprintf "\"%s\"" a3.cu_filename ++ text ";" ++ line ++
  text "package = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_qualified_ident () elt)) a3.package
  ++ text ";" ++ line ++
  text "imports = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_import () elt))
                     a3.imports ++ text ";" ++ line ++
  text "decls = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_type_decl () elt)) a3.decls ++ unalign ++
  text "; }" 

and d_import () (a4 : import) =
  begin match a4 with
    SingleTypeImport a5 -> text "SingleTypeImport(" ++ break ++ 
                           d_qualified_ident () a5 ++ text ")"
  | TypeImportOnDemand a6 -> text "TypeImportOnDemand(" ++ break ++ 
                             d_qualified_ident () a6 ++ text ")"
  end

and d_class_decl () (a7 : class_decl) =
  text "{ " ++ align ++ text "c_decl_modifiers = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_modifier () elt))
                     a7.c_decl_modifiers ++ text ";" ++ line ++
  text "c_name = " ++ break ++ d_ident () a7.c_name ++ text ";" ++ line ++
  text "c_extends = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_typ () elt)) a7.c_extends
  ++ text ";" ++ line ++
  text "c_implements = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_typ () elt))
                     a7.c_implements ++ text ";" ++ line ++
  text "c_body = " ++ break ++ d_class_body () a7.c_body ++ unalign ++
  text "; }" 

and d_interface_decl () (a8 : interface_decl) =
  text "{ " ++ align ++ text "i_decl_modifiers = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_modifier () elt))
                     a8.i_decl_modifiers ++ text ";" ++ line ++
  text "i_name = " ++ break ++ d_ident () a8.i_name ++ text ";" ++ line ++
  text "i_extends = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_typ () elt))
                     a8.i_extends ++ text ";" ++ line ++
  text "i_body = " ++ break ++ d_interface_body () a8.i_body ++ unalign ++
  text "; }" 

and d_modifier () (a9 : modifier) =
  begin match a9 with
    Static -> text "Static"
  | Abstract -> text "Abstract"
  | Final -> text "Final"
  | Public -> text "Public"
  | Protected -> text "Protected"
  | Private -> text "Private"
  | Strictfp -> text "Strictfp"
  | Native -> text "Native"
  | Synchronized -> text "Synchronized"
  | Transient -> text "Transient"
  | Volatile -> text "Volatile"
  end

and d_type_decl () (a10 : type_decl) =
  begin match a10 with
    ClassDecl a11 -> text "ClassDecl(" ++ break ++  d_class_decl () a11
                     ++ text ")"
  | InterfaceDecl a12 -> text "InterfaceDecl(" ++ break ++ 
                         d_interface_decl () a12 ++ text ")"
  end

and d_class_body () (a13 : class_body) =
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_class_body_decl () elt)) a13

and d_class_body_decl () (a14 : class_body_decl) =
  begin match a14 with
    ClassMemberDecl a15 -> text "ClassMemberDecl(" ++ break ++ 
                           d_class_member_decl () a15 ++ text ")"
  | InstanceInitializer a16 -> text "InstanceInitializer(" ++ break ++ 
                               d_block () a16 ++ text ")"
  | StaticInitializer a17 -> text "StaticInitializer(" ++ break ++ 
                             d_block () a17 ++ text ")"
  | ConstructorDecl a18 -> text "ConstructorDecl(" ++ break ++ 
                           d_constructor_decl () a18 ++ text ")"
  end

and d_class_member_decl () (a19 : class_member_decl) =
  begin match a19 with
    FieldDecl a20 -> text "FieldDecl(" ++ break ++  d_field_decl () a20
                     ++ text ")"
  | MethodDecl a21 -> text "MethodDecl(" ++ break ++  d_method_decl () a21
                      ++ text ")"
  | ClassOrInterfaceDecl a22 -> text "ClassOrInterfaceDecl(" ++ break ++ 
                                d_type_decl () a22 ++ text ")"
  | NopClassMemberDecl -> text "NopClassMemberDecl"
  end

and d_interface_body () (a23 : interface_body) =
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_class_member_decl () elt)) a23

and d_field_decl () (a24 : field_decl) =
  text "{ " ++ align ++ text "fd_modifiers = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_modifier () elt))
                     a24.fd_modifiers ++ text ";" ++ line ++
  text "fd_type = " ++ break ++ d_typ () a24.fd_type ++ text ";" ++ line ++
  text "fd_vd = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_variable_declarator () elt)) a24.fd_vd
  ++ unalign ++ text "; }" 

and d_variable_declarator () (a25 : variable_declarator) =
  text "{ " ++ align ++ text "vd_name = " ++ break ++ d_ident () a25.vd_name
  ++ text ";" ++ line ++
  text "vd_brackets = " ++ break ++ dprintf "%d" a25.vd_brackets ++ text ";"
  ++ line ++
  text "vd_init = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_variable_initializer () elt))
                   a25.vd_init ++ unalign ++ text "; }" 

and d_variable_initializer () (a26 : variable_initializer) =
  begin match a26 with
    VI_ExpInit a27 -> text "VI_ExpInit(" ++ break ++  d_expn () a27 ++ text ")"
  | VI_ArrayInit a28 -> text "VI_ArrayInit(" ++ break ++ 
                        d_array_initializer () a28 ++ text ")"
  end

and d_array_initializer () (a29 : array_initializer) =
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_variable_initializer () elt)) a29

and d_typ () (a30 : typ) =
  begin match a30 with
    Primitive a31 -> text "Primitive(" ++ break ++  d_primitive_type () a31
                     ++ text ")"
  | Reference a32 -> text "Reference(" ++ break ++  d_reference_type () a32
                     ++ text ")"
  end

and d_primitive_type () (a33 : primitive_type) =
  begin match a33 with
    Boolean -> text "Boolean"
  | Char -> text "Char"
  | Byte -> text "Byte"
  | Short -> text "Short"
  | Int -> text "Int"
  | Long -> text "Long"
  | Float -> text "Float"
  | Double -> text "Double"
  end

and d_reference_type () (a34 : reference_type) =
  begin match a34 with
    ClassOrInterfaceType a35 -> text "ClassOrInterfaceType(" ++ break ++ 
                                d_qualified_ident () a35 ++ text ")"
  | ArrayType a36 -> text "ArrayType(" ++ break ++  d_typ () a36 ++ text ")"
  end

and d_type_or_void () (a37 : type_or_void) =
  begin match a37 with
    Type a38 -> text "Type(" ++ break ++  d_typ () a38 ++ text ")"
  | Void -> text "Void"
  end

and d_constructor_decl () (a39 : constructor_decl) =
  text "{ " ++ align ++ text "cons_modifiers = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_modifier () elt))
                     a39.cons_modifiers ++ text ";" ++ line ++
  text "cons_name = " ++ break ++ d_ident () a39.cons_name ++ text ";"
  ++ line ++
  text "cons_formals = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_formal () elt))
                     a39.cons_formals ++ text ";" ++ line ++
  text "cons_throws = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_qualified_ident () elt)) a39.cons_throws
  ++ text ";" ++ line ++
  text "cons_body = " ++ break ++ d_block () a39.cons_body ++ unalign ++
  text "; }" 

and d_method_decl () (a40 : method_decl) =
  text "{ " ++ align ++ text "md_modifiers = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_modifier () elt))
                     a40.md_modifiers ++ text ";" ++ line ++
  text "md_type = " ++ break ++ d_type_or_void () a40.md_type ++ text ";"
  ++ line ++
  text "md_name = " ++ break ++ d_ident () a40.md_name ++ text ";" ++ line ++
  text "md_formals = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_formal () elt))
                     a40.md_formals ++ text ";" ++ line ++
  text "md_throws = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_qualified_ident () elt)) a40.md_throws
  ++ text ";" ++ line ++
  text "md_body = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_block () elt)) a40.md_body
  ++ unalign ++ text "; }" 

and d_formal () (a41 : formal) =
  text "{ " ++ align ++ text "f_final = " ++ break ++ dprintf "%b" a41.f_final
  ++ text ";" ++ line ++
  text "f_type = " ++ break ++ d_typ () a41.f_type ++ text ";" ++ line ++
  text "f_name = " ++ break ++ d_ident () a41.f_name ++ text ";" ++ line ++
  text "f_brackets = " ++ break ++ dprintf "%d" a41.f_brackets ++ unalign ++
  text "; }" 

and d_block () (a42 : block) =
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_block_statement () elt)) a42

and d_block_statement () (a43 : block_statement) =
  begin match a43 with
    LocalVariableDecl a44 -> text "LocalVariableDecl(" ++ break ++ 
                             d_local_variable_decl () a44 ++ text ")"
  | BlockClassDecl a45 -> text "BlockClassDecl(" ++ break ++ 
                          d_class_decl () a45 ++ text ")"
  | Statement a46 -> text "Statement(" ++ break ++  d_stmtn () a46 ++ text ")"
  end

and d_local_variable_decl () (a47 : local_variable_decl) =
  text "{ " ++ align ++ text "lv_final = " ++ break ++ dprintf "%b" a47.lv_final
  ++ text ";" ++ line ++
  text "lv_type = " ++ break ++ d_typ () a47.lv_type ++ text ";" ++ line ++
  text "lv_vd = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_variable_declarator () elt)) a47.lv_vd
  ++ unalign ++ text "; }" 

and d_stmtn () (a48 : stmtn) =
  let a49 = (a48).node in
   d_stmt () a49

and d_stmt () (a50 : stmt) =
  begin match a50 with
    Labeled(a51,a52) -> text "Labeled(" ++ break ++  d_ident () a51
                        ++ text "," ++ d_stmtn () a52 ++ text ")"
  | IfThen(a53,a54) -> text "IfThen(" ++ break ++  d_expn () a53 ++ text "," ++
                       d_stmtn () a54 ++ text ")"
  | IfThenElse(a55,a56,a57) -> text "IfThenElse(" ++ break ++  d_expn () a55
                               ++ text "," ++ d_stmtn () a56 ++ text "," ++
                               d_stmtn () a57 ++ text ")"
  | ExpStmt a58 -> text "ExpStmt(" ++ break ++  d_exp_stmt () a58 ++ text ")"
  | Switch(a59,a60) -> text "Switch(" ++ break ++  d_expn () a59 ++ text "," ++
                       d_switch_block () a60 ++ text ")"
  | While(a61,a62) -> text "While(" ++ break ++  d_expn () a61 ++ text "," ++
                      d_stmtn () a62 ++ text ")"
  | Do(a63,a64) -> text "Do(" ++ break ++  d_stmtn () a63 ++ text "," ++
                   d_expn () a64 ++ text ")"
  | For a65 -> text "For(" ++ break ++  d_for_stmt () a65 ++ text ")"
  | BlockStmt a66 -> text "BlockStmt(" ++ break ++  d_block () a66 ++ text ")"
  | EmptyStmt -> text "EmptyStmt"
  | Break a67 -> text "Break(" ++ break ++ 
                 dprintf "@[%a@]" (docOpt (fun () elt -> d_ident () elt)) a67
                 ++ text ")"
  | Continue a68 -> text "Continue(" ++ break ++ 
                    dprintf "@[%a@]" (docOpt (fun () elt -> d_ident () elt)) a68
                    ++ text ")"
  | Return a69 -> text "Return(" ++ break ++ 
                  dprintf "@[%a@]" (docOpt (fun () elt -> d_expn () elt)) a69
                  ++ text ")"
  | Throw a70 -> text "Throw(" ++ break ++  d_expn () a70 ++ text ")"
  | SynchronizedExp(a71,a72) -> text "SynchronizedExp(" ++ break ++ 
                                d_expn () a71 ++ text "," ++ d_block () a72
                                ++ text ")"
  | Try a73 -> text "Try(" ++ break ++  d_try_stmt () a73 ++ text ")"
  end

and d_try_stmt () (a74 : try_stmt) =
  text "{ " ++ align ++ text "try_block = " ++ break ++ d_block () a74.try_block
  ++ text ";" ++ line ++
  text "try_catch = " ++ break ++ d_catches () a74.try_catch ++ text ";"
  ++ line ++
  text "try_finally = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_block () elt)) a74.try_finally
  ++ unalign ++ text "; }" 

and d_catches () (a75 : catches) =
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> let (a76,
                     a77) = elt in
                       text "(" ++ d_formal () a76 ++ text "," ++ d_block () a77
                       ++ text ")")) a75

and d_for_stmt () (a78 : for_stmt) =
  text "{ " ++ align ++ text "for_init = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_for_init () elt)) a78.for_init
  ++ text ";" ++ line ++
  text "for_guard = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_expn () elt)) a78.for_guard
  ++ text ";" ++ line ++
  text "for_update = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_esn () elt))
                     a78.for_update ++ text ";" ++ line ++
  text "for_body = " ++ break ++ d_stmtn () a78.for_body ++ unalign ++
  text "; }" 

and d_for_init () (a79 : for_init) =
  begin match a79 with
    ForInitStmtExp a80 -> text "ForInitStmtExp(" ++ break ++ 
                          dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                                             (fun elt -> d_esn () elt)) a80
                          ++ text ")"
  | ForInitVarDecl a81 -> text "ForInitVarDecl(" ++ break ++ 
                          d_local_variable_decl () a81 ++ text ")"
  end

and d_switch_block () (a82 : switch_block) =
  text "{ " ++ align ++ text "sb_stmt_groups = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_switch_block_stmt_group () elt))
                     a82.sb_stmt_groups ++ unalign ++ text "; }" 

and d_switch_block_stmt_group () (a83 : switch_block_stmt_group) =
  text "{ " ++ align ++ text "sbsg_labels = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_switch_label () elt)) a83.sbsg_labels
  ++ text ";" ++ line ++
  text "sbsg_stmts = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                     (fun elt -> d_block_statement () elt)) a83.sbsg_stmts
  ++ unalign ++ text "; }" 

and d_switch_label () (a84 : switch_label) =
  begin match a84 with
    Default -> text "Default"
  | Case a85 -> text "Case(" ++ break ++  d_expn () a85 ++ text ")"
  end

and d_esn () (a86 : esn) =
  let a87 = (a86).node in
   d_exp_stmt () a87

and d_exp_stmt () (a88 : exp_stmt) =
  begin match a88 with
    Assignment(a89,a90,a91) -> text "Assignment(" ++ break ++  d_aop () a89
                               ++ text "," ++ d_expn () a90 ++ text "," ++
                               d_expn () a91 ++ text ")"
  | PreOp(a92,a93) -> text "PreOp(" ++ break ++  d_preop () a92 ++ text "," ++
                      d_expn () a93 ++ text ")"
  | PostOp(a94,a95) -> text "PostOp(" ++ break ++  d_expn () a94 ++ text "," ++
                       d_postop () a95 ++ text ")"
  | MethodInvoc(a96,a97) -> text "MethodInvoc(" ++ break ++  d_ident () a96
                            ++ text "," ++
                            dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                                               (fun elt -> d_expn () elt)) a97
                            ++ text ")"
  | MethodInvocExp(a98,a99,a100) -> text "MethodInvocExp(" ++ break ++ 
                                    d_expn () a98 ++ text "," ++ d_ident () a99
                                    ++ text "," ++
                                    dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                                                       (fun elt -> d_expn () elt))
                                                       a100 ++ text ")"
  | MethodInvocSuper(a101,a102) -> text "MethodInvocSuper(" ++ break ++ 
                                   dprintf "@[%a@]" (docOpt (fun () elt -> d_ident () elt))
                                                    a101 ++ text "," ++
                                   dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                                                      (fun elt -> d_expn () elt))
                                                      a102 ++ text ")"
  | MethodInvocThis a103 -> text "MethodInvocThis(" ++ break ++ 
                            dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                                               (fun elt -> d_expn () elt)) a103
                            ++ text ")"
  | MethodInvocClassSuper(a104,a105,a106) -> text "MethodInvocClassSuper(" ++ break ++ 
                                             d_qualified_ident () a104
                                             ++ text "," ++ d_ident () a105
                                             ++ text "," ++
                                             dprintf "[@[%a@]]" (docList (chr ';' ++ break)
                                                                (fun elt -> d_expn () elt))
                                                                a106 ++ text ")"
  | ClassInstanceCreationType(a107,a108) -> text "ClassInstanceCreationType(" ++ break ++ 
                                            d_typ () a107 ++ text "," ++
                                            d_class_instance_creation () a108
                                            ++ text ")"
  | ClassInstanceCreationExp(a109,a110,a111) -> text "ClassInstanceCreationExp(" ++ break ++ 
                                                d_expn () a109 ++ text "," ++
                                                d_ident () a110 ++ text "," ++
                                                d_class_instance_creation () a111
                                                ++ text ")"
  end

and d_aop () (a112 : aop) =
  begin match a112 with
    Assign -> text "Assign"
  | PlusAssign -> text "PlusAssign"
  | MinusAssign -> text "MinusAssign"
  | StarAssign -> text "StarAssign"
  | SlashAssign -> text "SlashAssign"
  | AndAssign -> text "AndAssign"
  | OrAssign -> text "OrAssign"
  | XorAssign -> text "XorAssign"
  | RemAssign -> text "RemAssign"
  | LShiftAssign -> text "LShiftAssign"
  | RSignedShiftAssign -> text "RSignedShiftAssign"
  | RUnsignedShiftAssign -> text "RUnsignedShiftAssign"
  end

and d_preop () (a113 : preop) =
  begin match a113 with
    PreInc -> text "PreInc"
  | PreDec -> text "PreDec"
  | Bang -> text "Bang"
  | Tilde -> text "Tilde"
  | PrePlus -> text "PrePlus"
  | PreMinus -> text "PreMinus"
  end

and d_postop () (a114 : postop) =
  begin match a114 with
    PostInc -> text "PostInc"
  | PostDec -> text "PostDec"
  end

and d_class_instance_creation () (a115 : class_instance_creation) =
  text "{ " ++ align ++ text "cic_args = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_expn () elt))
                     a115.cic_args ++ text ";" ++ line ++
  text "cic_body = " ++ break ++
  dprintf "@[%a@]" (docOpt (fun () elt -> d_class_body () elt)) a115.cic_body
  ++ unalign ++ text "; }" 

and d_expn () (a116 : expn) =
  let a117 = (a116).node in
   d_exp () a117

and d_exp () (a118 : exp) =
  begin match a118 with
    Literal a119 -> text "Literal(" ++ break ++  d_literal () a119 ++ text ")"
  | TypeDotClass a120 -> text "TypeDotClass(" ++ break ++  d_typ () a120
                         ++ text ")"
  | VoidDotClass -> text "VoidDotClass"
  | This -> text "This"
  | ClassNameDotThis a121 -> text "ClassNameDotThis(" ++ break ++ 
                             d_qualified_ident () a121 ++ text ")"
  | ES a122 -> text "ES(" ++ break ++  d_exp_stmt () a122 ++ text ")"
  | ArrayCreation a123 -> text "ArrayCreation(" ++ break ++ 
                          d_array_creation () a123 ++ text ")"
  | ArrayCreationInit a124 -> text "ArrayCreationInit(" ++ break ++ 
                              d_array_creation_init () a124 ++ text ")"
  | Field(a125,a126) -> text "Field(" ++ break ++  d_expn () a125 ++ text "," ++
                        d_ident () a126 ++ text ")"
  | SuperField a127 -> text "SuperField(" ++ break ++  d_ident () a127
                       ++ text ")"
  | ClassSuperField(a128,a129) -> text "ClassSuperField(" ++ break ++ 
                                  d_qualified_ident () a128 ++ text "," ++
                                  d_ident () a129 ++ text ")"
  | ArrayAccess(a130,a131) -> text "ArrayAccess(" ++ break ++  d_expn () a130
                              ++ text "," ++ d_expn () a131 ++ text ")"
  | Cast(a132,a133) -> text "Cast(" ++ break ++  d_typ () a132 ++ text "," ++
                       d_expn () a133 ++ text ")"
  | BinOp(a134,a135,a136) -> text "BinOp(" ++ break ++  d_bop () a134
                             ++ text "," ++ d_expn () a135 ++ text "," ++
                             d_expn () a136 ++ text ")"
  | InstanceOf(a137,a138) -> text "InstanceOf(" ++ break ++  d_expn () a137
                             ++ text "," ++ d_typ () a138 ++ text ")"
  | Conditional(a139,a140,a141) -> text "Conditional(" ++ break ++ 
                                   d_expn () a139 ++ text "," ++ d_expn () a140
                                   ++ text "," ++ d_expn () a141 ++ text ")"
  | ExpressionName a142 -> text "ExpressionName(" ++ break ++ 
                           d_qualified_ident () a142 ++ text ")"
  end

and d_bop () (a143 : bop) =
  begin match a143 with
    SC_Or -> text "SC_Or"
  | SC_And -> text "SC_And"
  | Bit_Or -> text "Bit_Or"
  | Bit_Xor -> text "Bit_Xor"
  | Bit_And -> text "Bit_And"
  | Eq -> text "Eq"
  | Ne -> text "Ne"
  | Gt -> text "Gt"
  | Lt -> text "Lt"
  | Ge -> text "Ge"
  | Le -> text "Le"
  | LShift -> text "LShift"
  | RSignedShift -> text "RSignedShift"
  | RUnsignedShift -> text "RUnsignedShift"
  | Plus -> text "Plus"
  | Minus -> text "Minus"
  | Star -> text "Star"
  | Slash -> text "Slash"
  | Rem -> text "Rem"
  end

and d_array_creation () (a144 : array_creation) =
  text "{ " ++ align ++ text "ac_type = " ++ break ++ d_typ () a144.ac_type
  ++ text ";" ++ line ++
  text "ac_dim_expr = " ++ break ++
  dprintf "[@[%a@]]" (docList (chr ';' ++ break) (fun elt -> d_expn () elt))
                     a144.ac_dim_expr ++ text ";" ++ line ++
  text "ac_dims = " ++ break ++ dprintf "%d" a144.ac_dims ++ unalign ++
  text "; }" 

and d_array_creation_init () (a145 : array_creation_init) =
  text "{ " ++ align ++ text "aci_type = " ++ break ++ d_typ () a145.aci_type
  ++ text ";" ++ line ++
  text "aci_dims = " ++ break ++ dprintf "%d" a145.aci_dims ++ text ";"
  ++ line ++
  text "aci_init = " ++ break ++ d_array_initializer () a145.aci_init
  ++ unalign ++ text "; }" 

and d_literal () (a146 : literal) =
  begin match a146 with
    IntLit a147 -> text "IntLit(" ++ break ++  dprintf "\"%s\"" a147 ++ text ")"
  | FloatLit a148 -> text "FloatLit(" ++ break ++  dprintf "\"%s\"" a148
                     ++ text ")"
  | CharLit a149 -> text "CharLit(" ++ break ++  dprintf "\"%s\"" a149
                    ++ text ")"
  | StringLit a150 -> text "StringLit(" ++ break ++  dprintf "\"%s\"" a150
                      ++ text ")"
  | True -> text "True"
  | False -> text "False"
  | Null -> text "Null"
  end

(** Different visiting actions. *)
type 'a visitAction = 
    SkipChildren                        (** Do not visit the children. Return 
                                            the node as it is. *)
  | DoChildren                          (** Continue with the children of this 
                                            node. Rebuild the node on return 
                                            if any of the children changes 
                                            (use == test) *)
  | ChangeTo of 'a                      (** Replace the expression with the 
                                            given one *)
  | ChangeDoChildrenPost of 'a * ('a -> 'a) (** First consider that the entire 
                                           exp is replaced by the first 
                                           parameter. Then continue with 
                                           the children. On return rebuild 
                                           the node if any of the children 
                                           has changed and then apply the 
                                           function on the node *)
class type jabsVisitor = object
  method v_ident : (ident) -> (ident) visitAction
  method v_qualified_ident : (qualified_ident) -> (qualified_ident) visitAction
  method v_node : ('a node) -> ('a node) visitAction
  method v_compilation_unit : (compilation_unit) -> (compilation_unit) visitAction
  method v_import : (import) -> (import) visitAction
  method v_class_decl : (class_decl) -> (class_decl) visitAction
  method v_interface_decl : (interface_decl) -> (interface_decl) visitAction
  method v_modifier : (modifier) -> (modifier) visitAction
  method v_type_decl : (type_decl) -> (type_decl) visitAction
  method v_class_body : (class_body) -> (class_body) visitAction
  method v_class_body_decl : (class_body_decl) -> (class_body_decl) visitAction
  method v_class_member_decl : (class_member_decl) -> (class_member_decl) visitAction
  method v_interface_body : (interface_body) -> (interface_body) visitAction
  method v_field_decl : (field_decl) -> (field_decl) visitAction
  method v_variable_declarator : (variable_declarator) -> (variable_declarator) visitAction
  method v_variable_initializer : (variable_initializer) -> (variable_initializer) visitAction
  method v_array_initializer : (array_initializer) -> (array_initializer) visitAction
  method v_typ : (typ) -> (typ) visitAction
  method v_primitive_type : (primitive_type) -> (primitive_type) visitAction
  method v_reference_type : (reference_type) -> (reference_type) visitAction
  method v_type_or_void : (type_or_void) -> (type_or_void) visitAction
  method v_constructor_decl : (constructor_decl) -> (constructor_decl) visitAction
  method v_method_decl : (method_decl) -> (method_decl) visitAction
  method v_formal : (formal) -> (formal) visitAction
  method v_block : (block) -> (block) visitAction
  method v_block_statement : (block_statement) -> (block_statement) visitAction
  method v_local_variable_decl : (local_variable_decl) -> (local_variable_decl) visitAction
  method v_stmtn : (stmtn) -> (stmtn) visitAction
  method v_stmt : (stmt) -> (stmt) visitAction
  method v_try_stmt : (try_stmt) -> (try_stmt) visitAction
  method v_catches : (catches) -> (catches) visitAction
  method v_for_stmt : (for_stmt) -> (for_stmt) visitAction
  method v_for_init : (for_init) -> (for_init) visitAction
  method v_switch_block : (switch_block) -> (switch_block) visitAction
  method v_switch_block_stmt_group : (switch_block_stmt_group) -> (switch_block_stmt_group) visitAction
  method v_switch_label : (switch_label) -> (switch_label) visitAction
  method v_esn : (esn) -> (esn) visitAction
  method v_exp_stmt : (exp_stmt) -> (exp_stmt) visitAction
  method v_aop : (aop) -> (aop) visitAction
  method v_preop : (preop) -> (preop) visitAction
  method v_postop : (postop) -> (postop) visitAction
  method v_class_instance_creation : (class_instance_creation) -> (class_instance_creation) visitAction
  method v_expn : (expn) -> (expn) visitAction
  method v_exp : (exp) -> (exp) visitAction
  method v_bop : (bop) -> (bop) visitAction
  method v_array_creation : (array_creation) -> (array_creation) visitAction
  method v_array_creation_init : (array_creation_init) -> (array_creation_init) visitAction
  method v_literal : (literal) -> (literal) visitAction
end
class nopJabsVisitor : jabsVisitor = object
  method v_ident : ((ident) -> ((ident) visitAction)) = fun _ -> DoChildren
  method v_qualified_ident : ((qualified_ident) -> ((qualified_ident) visitAction)) =
  fun _ -> DoChildren
  method v_node : 'a. (('a node) -> (('a node) visitAction)) =
  fun _ -> DoChildren
  method v_compilation_unit : ((compilation_unit) -> ((compilation_unit) visitAction)) =
  fun _ -> DoChildren
  method v_import : ((import) -> ((import) visitAction)) = fun _ -> DoChildren
  method v_class_decl : ((class_decl) -> ((class_decl) visitAction)) =
  fun _ -> DoChildren
  method v_interface_decl : ((interface_decl) -> ((interface_decl) visitAction)) =
  fun _ -> DoChildren
  method v_modifier : ((modifier) -> ((modifier) visitAction)) =
  fun _ -> DoChildren
  method v_type_decl : ((type_decl) -> ((type_decl) visitAction)) =
  fun _ -> DoChildren
  method v_class_body : ((class_body) -> ((class_body) visitAction)) =
  fun _ -> DoChildren
  method v_class_body_decl : ((class_body_decl) -> ((class_body_decl) visitAction)) =
  fun _ -> DoChildren
  method v_class_member_decl : ((class_member_decl) -> ((class_member_decl) visitAction)) =
  fun _ -> DoChildren
  method v_interface_body : ((interface_body) -> ((interface_body) visitAction)) =
  fun _ -> DoChildren
  method v_field_decl : ((field_decl) -> ((field_decl) visitAction)) =
  fun _ -> DoChildren
  method v_variable_declarator : ((variable_declarator) -> ((variable_declarator) visitAction)) =
  fun _ -> DoChildren
  method v_variable_initializer : ((variable_initializer) -> ((variable_initializer) visitAction)) =
  fun _ -> DoChildren
  method v_array_initializer : ((array_initializer) -> ((array_initializer) visitAction)) =
  fun _ -> DoChildren
  method v_typ : ((typ) -> ((typ) visitAction)) = fun _ -> DoChildren
  method v_primitive_type : ((primitive_type) -> ((primitive_type) visitAction)) =
  fun _ -> DoChildren
  method v_reference_type : ((reference_type) -> ((reference_type) visitAction)) =
  fun _ -> DoChildren
  method v_type_or_void : ((type_or_void) -> ((type_or_void) visitAction)) =
  fun _ -> DoChildren
  method v_constructor_decl : ((constructor_decl) -> ((constructor_decl) visitAction)) =
  fun _ -> DoChildren
  method v_method_decl : ((method_decl) -> ((method_decl) visitAction)) =
  fun _ -> DoChildren
  method v_formal : ((formal) -> ((formal) visitAction)) = fun _ -> DoChildren
  method v_block : ((block) -> ((block) visitAction)) = fun _ -> DoChildren
  method v_block_statement : ((block_statement) -> ((block_statement) visitAction)) =
  fun _ -> DoChildren
  method v_local_variable_decl : ((local_variable_decl) -> ((local_variable_decl) visitAction)) =
  fun _ -> DoChildren
  method v_stmtn : ((stmtn) -> ((stmtn) visitAction)) = fun _ -> DoChildren
  method v_stmt : ((stmt) -> ((stmt) visitAction)) = fun _ -> DoChildren
  method v_try_stmt : ((try_stmt) -> ((try_stmt) visitAction)) =
  fun _ -> DoChildren
  method v_catches : ((catches) -> ((catches) visitAction)) =
  fun _ -> DoChildren
  method v_for_stmt : ((for_stmt) -> ((for_stmt) visitAction)) =
  fun _ -> DoChildren
  method v_for_init : ((for_init) -> ((for_init) visitAction)) =
  fun _ -> DoChildren
  method v_switch_block : ((switch_block) -> ((switch_block) visitAction)) =
  fun _ -> DoChildren
  method v_switch_block_stmt_group : ((switch_block_stmt_group) -> ((switch_block_stmt_group) visitAction)) =
  fun _ -> DoChildren
  method v_switch_label : ((switch_label) -> ((switch_label) visitAction)) =
  fun _ -> DoChildren
  method v_esn : ((esn) -> ((esn) visitAction)) = fun _ -> DoChildren
  method v_exp_stmt : ((exp_stmt) -> ((exp_stmt) visitAction)) =
  fun _ -> DoChildren
  method v_aop : ((aop) -> ((aop) visitAction)) = fun _ -> DoChildren
  method v_preop : ((preop) -> ((preop) visitAction)) = fun _ -> DoChildren
  method v_postop : ((postop) -> ((postop) visitAction)) = fun _ -> DoChildren
  method v_class_instance_creation : ((class_instance_creation) -> ((class_instance_creation) visitAction)) =
  fun _ -> DoChildren
  method v_expn : ((expn) -> ((expn) visitAction)) = fun _ -> DoChildren
  method v_exp : ((exp) -> ((exp) visitAction)) = fun _ -> DoChildren
  method v_bop : ((bop) -> ((bop) visitAction)) = fun _ -> DoChildren
  method v_array_creation : ((array_creation) -> ((array_creation) visitAction)) =
  fun _ -> DoChildren
  method v_array_creation_init : ((array_creation_init) -> ((array_creation_init) visitAction)) =
  fun _ -> DoChildren
  method v_literal : ((literal) -> ((literal) visitAction)) =
  fun _ -> DoChildren
end

(* visit all the nodes in a type expression *)
let doVisit (vis: 'b)
            (startvisit: 'a -> 'a visitAction)
            (children: 'b -> 'a -> 'a)
            (node: 'a) : 'a =
  let action = startvisit node in
  match action with
    SkipChildren -> node
  | ChangeTo node' -> node'
  | _ -> (* DoChildren and ChangeDoChildrenPost *)
      let nodepre = match action with
        ChangeDoChildrenPost (node', _) -> node'
      | _ -> node                                                                     in
      let nodepost = children vis nodepre in
      match action with
        ChangeDoChildrenPost (_, f) -> f nodepost
      | _ -> nodepost
let rec visit_ident (vis : jabsVisitor) v =
  doVisit vis vis#v_ident visit_children_ident v
and visit_children_ident (vis : jabsVisitor) v =
  v (* no string visitor *)
and visit_qualified_ident (vis : jabsVisitor) v =
  doVisit vis vis#v_qualified_ident visit_children_qualified_ident v
and visit_children_qualified_ident (vis : jabsVisitor) v =
  List.map (fun v -> visit_ident vis v) v
and visit_node (vis : jabsVisitor) v =
  doVisit vis vis#v_node visit_children_node v
and visit_children_node (vis : jabsVisitor) v =
  let tmp_v_node = v.node in
  let tmp_v_span = v.span (* no Jabsutil.span visitor *) in
  if (tmp_v_node == v.node && tmp_v_span == v.span) then v
  else { node = tmp_v_node;  span = tmp_v_span; }
and visit_compilation_unit (vis : jabsVisitor) v =
  doVisit vis vis#v_compilation_unit visit_children_compilation_unit v
and visit_children_compilation_unit (vis : jabsVisitor) v =
  let tmp_v_cu_filename = v.cu_filename (* no string visitor *) in
  let tmp_v_package = match v.package with None -> v.package
                       | Some(x) -> (let tmp_x = visit_qualified_ident vis x in
                      if (tmp_x == x) then v.package else (Some((tmp_x)))) in
  let tmp_v_imports = List.map (fun v -> visit_import vis v) v.imports in
  let tmp_v_decls = List.map (fun v -> visit_type_decl vis v) v.decls in
  if (tmp_v_cu_filename == v.cu_filename && tmp_v_package == v.package &&
  tmp_v_imports == v.imports && tmp_v_decls == v.decls) then v
  else { cu_filename = tmp_v_cu_filename;  package = tmp_v_package; 
        imports = tmp_v_imports;  decls = tmp_v_decls; }
and visit_import (vis : jabsVisitor) v =
  doVisit vis vis#v_import visit_children_import v
and visit_children_import (vis : jabsVisitor) v =
  begin match v with
    SingleTypeImport a151 -> let tmp_a151 = visit_qualified_ident vis a151 in
                             if (tmp_a151 == a151) then v
                             else (SingleTypeImport(tmp_a151))
  | TypeImportOnDemand a152 -> let tmp_a152 = visit_qualified_ident vis a152 in
                               if (tmp_a152 == a152) then v
                               else (TypeImportOnDemand(tmp_a152))
  end
and visit_class_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_class_decl visit_children_class_decl v
and visit_children_class_decl (vis : jabsVisitor) v =
  let tmp_v_c_decl_modifiers = List.map (fun v -> visit_modifier vis v) v.c_decl_modifiers in
  let tmp_v_c_name = visit_ident vis v.c_name in
  let tmp_v_c_extends = match v.c_extends with None -> v.c_extends
                         | Some(x) -> (let tmp_x = visit_typ vis x in
                        if (tmp_x == x) then v.c_extends else (Some((tmp_x)))) in
  let tmp_v_c_implements = List.map (fun v -> visit_typ vis v) v.c_implements in
  let tmp_v_c_body = visit_class_body vis v.c_body in
  if (tmp_v_c_decl_modifiers == v.c_decl_modifiers && tmp_v_c_name == v.c_name
  && tmp_v_c_extends == v.c_extends && tmp_v_c_implements == v.c_implements &&
  tmp_v_c_body == v.c_body) then v
  else { c_decl_modifiers = tmp_v_c_decl_modifiers;  c_name = tmp_v_c_name; 
        c_extends = tmp_v_c_extends;  c_implements = tmp_v_c_implements; 
        c_body = tmp_v_c_body; }
and visit_interface_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_interface_decl visit_children_interface_decl v
and visit_children_interface_decl (vis : jabsVisitor) v =
  let tmp_v_i_decl_modifiers = List.map (fun v -> visit_modifier vis v) v.i_decl_modifiers in
  let tmp_v_i_name = visit_ident vis v.i_name in
  let tmp_v_i_extends = List.map (fun v -> visit_typ vis v) v.i_extends in
  let tmp_v_i_body = visit_interface_body vis v.i_body in
  if (tmp_v_i_decl_modifiers == v.i_decl_modifiers && tmp_v_i_name == v.i_name
  && tmp_v_i_extends == v.i_extends && tmp_v_i_body == v.i_body) then v
  else { i_decl_modifiers = tmp_v_i_decl_modifiers;  i_name = tmp_v_i_name; 
        i_extends = tmp_v_i_extends;  i_body = tmp_v_i_body; }
and visit_modifier (vis : jabsVisitor) v =
  doVisit vis vis#v_modifier visit_children_modifier v
and visit_children_modifier (vis : jabsVisitor) v =
  begin match v with
    Static -> v
  | Abstract -> v
  | Final -> v
  | Public -> v
  | Protected -> v
  | Private -> v
  | Strictfp -> v
  | Native -> v
  | Synchronized -> v
  | Transient -> v
  | Volatile -> v
  end
and visit_type_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_type_decl visit_children_type_decl v
and visit_children_type_decl (vis : jabsVisitor) v =
  begin match v with
    ClassDecl a153 -> let tmp_a153 = visit_class_decl vis a153 in
                      if (tmp_a153 == a153) then v else (ClassDecl(tmp_a153))
  | InterfaceDecl a154 -> let tmp_a154 = visit_interface_decl vis a154 in
                          if (tmp_a154 == a154) then v
                          else (InterfaceDecl(tmp_a154))
  end
and visit_class_body (vis : jabsVisitor) v =
  doVisit vis vis#v_class_body visit_children_class_body v
and visit_children_class_body (vis : jabsVisitor) v =
  List.map (fun v -> visit_class_body_decl vis v) v
and visit_class_body_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_class_body_decl visit_children_class_body_decl v
and visit_children_class_body_decl (vis : jabsVisitor) v =
  begin match v with
    ClassMemberDecl a155 -> let tmp_a155 = visit_class_member_decl vis a155 in
                            if (tmp_a155 == a155) then v
                            else (ClassMemberDecl(tmp_a155))
  | InstanceInitializer a156 -> let tmp_a156 = visit_block vis a156 in
                                if (tmp_a156 == a156) then v
                                else (InstanceInitializer(tmp_a156))
  | StaticInitializer a157 -> let tmp_a157 = visit_block vis a157 in
                              if (tmp_a157 == a157) then v
                              else (StaticInitializer(tmp_a157))
  | ConstructorDecl a158 -> let tmp_a158 = visit_constructor_decl vis a158 in
                            if (tmp_a158 == a158) then v
                            else (ConstructorDecl(tmp_a158))
  end
and visit_class_member_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_class_member_decl visit_children_class_member_decl v
and visit_children_class_member_decl (vis : jabsVisitor) v =
  begin match v with
    FieldDecl a159 -> let tmp_a159 = visit_field_decl vis a159 in
                      if (tmp_a159 == a159) then v else (FieldDecl(tmp_a159))
  | MethodDecl a160 -> let tmp_a160 = visit_method_decl vis a160 in
                       if (tmp_a160 == a160) then v else (MethodDecl(tmp_a160))
  | ClassOrInterfaceDecl a161 -> let tmp_a161 = visit_type_decl vis a161 in
                                 if (tmp_a161 == a161) then v
                                 else (ClassOrInterfaceDecl(tmp_a161))
  | NopClassMemberDecl -> v
  end
and visit_interface_body (vis : jabsVisitor) v =
  doVisit vis vis#v_interface_body visit_children_interface_body v
and visit_children_interface_body (vis : jabsVisitor) v =
  List.map (fun v -> visit_class_member_decl vis v) v
and visit_field_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_field_decl visit_children_field_decl v
and visit_children_field_decl (vis : jabsVisitor) v =
  let tmp_v_fd_modifiers = List.map (fun v -> visit_modifier vis v) v.fd_modifiers in
  let tmp_v_fd_type = visit_typ vis v.fd_type in
  let tmp_v_fd_vd = List.map (fun v -> visit_variable_declarator vis v) v.fd_vd in
  if (tmp_v_fd_modifiers == v.fd_modifiers && tmp_v_fd_type == v.fd_type &&
  tmp_v_fd_vd == v.fd_vd) then v
  else { fd_modifiers = tmp_v_fd_modifiers; 
        fd_type = tmp_v_fd_type;  fd_vd = tmp_v_fd_vd; }
and visit_variable_declarator (vis : jabsVisitor) v =
  doVisit vis vis#v_variable_declarator visit_children_variable_declarator v
and visit_children_variable_declarator (vis : jabsVisitor) v =
  let tmp_v_vd_name = visit_ident vis v.vd_name in
  let tmp_v_vd_brackets = v.vd_brackets (* no int visitor *) in
  let tmp_v_vd_init = match v.vd_init with None -> v.vd_init
                       | Some(x) -> (let tmp_x = visit_variable_initializer vis x in
                      if (tmp_x == x) then v.vd_init else (Some((tmp_x)))) in
  if (tmp_v_vd_name == v.vd_name && tmp_v_vd_brackets == v.vd_brackets &&
  tmp_v_vd_init == v.vd_init) then v
  else { vd_name = tmp_v_vd_name;  vd_brackets = tmp_v_vd_brackets; 
        vd_init = tmp_v_vd_init; }
and visit_variable_initializer (vis : jabsVisitor) v =
  doVisit vis vis#v_variable_initializer visit_children_variable_initializer v
and visit_children_variable_initializer (vis : jabsVisitor) v =
  begin match v with
    VI_ExpInit a162 -> let tmp_a162 = visit_expn vis a162 in
                       if (tmp_a162 == a162) then v else (VI_ExpInit(tmp_a162))
  | VI_ArrayInit a163 -> let tmp_a163 = visit_array_initializer vis a163 in
                         if (tmp_a163 == a163) then v
                         else (VI_ArrayInit(tmp_a163))
  end
and visit_array_initializer (vis : jabsVisitor) v =
  doVisit vis vis#v_array_initializer visit_children_array_initializer v
and visit_children_array_initializer (vis : jabsVisitor) v =
  List.map (fun v -> visit_variable_initializer vis v) v
and visit_typ (vis : jabsVisitor) v =
  doVisit vis vis#v_typ visit_children_typ v
and visit_children_typ (vis : jabsVisitor) v =
  begin match v with
    Primitive a164 -> let tmp_a164 = visit_primitive_type vis a164 in
                      if (tmp_a164 == a164) then v else (Primitive(tmp_a164))
  | Reference a165 -> let tmp_a165 = visit_reference_type vis a165 in
                      if (tmp_a165 == a165) then v else (Reference(tmp_a165))
  end
and visit_primitive_type (vis : jabsVisitor) v =
  doVisit vis vis#v_primitive_type visit_children_primitive_type v
and visit_children_primitive_type (vis : jabsVisitor) v =
  begin match v with
    Boolean -> v
  | Char -> v
  | Byte -> v
  | Short -> v
  | Int -> v
  | Long -> v
  | Float -> v
  | Double -> v
  end
and visit_reference_type (vis : jabsVisitor) v =
  doVisit vis vis#v_reference_type visit_children_reference_type v
and visit_children_reference_type (vis : jabsVisitor) v =
  begin match v with
    ClassOrInterfaceType a166 -> let tmp_a166 = visit_qualified_ident vis a166 in
                                 if (tmp_a166 == a166) then v
                                 else (ClassOrInterfaceType(tmp_a166))
  | ArrayType a167 -> let tmp_a167 = visit_typ vis a167 in
                      if (tmp_a167 == a167) then v else (ArrayType(tmp_a167))
  end
and visit_type_or_void (vis : jabsVisitor) v =
  doVisit vis vis#v_type_or_void visit_children_type_or_void v
and visit_children_type_or_void (vis : jabsVisitor) v =
  begin match v with
    Type a168 -> let tmp_a168 = visit_typ vis a168 in
                 if (tmp_a168 == a168) then v else (Type(tmp_a168))
  | Void -> v
  end
and visit_constructor_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_constructor_decl visit_children_constructor_decl v
and visit_children_constructor_decl (vis : jabsVisitor) v =
  let tmp_v_cons_modifiers = List.map (fun v -> visit_modifier vis v) v.cons_modifiers in
  let tmp_v_cons_name = visit_ident vis v.cons_name in
  let tmp_v_cons_formals = List.map (fun v -> visit_formal vis v) v.cons_formals in
  let tmp_v_cons_throws = List.map (fun v -> visit_qualified_ident vis v) v.cons_throws in
  let tmp_v_cons_body = visit_block vis v.cons_body in
  if (tmp_v_cons_modifiers == v.cons_modifiers && tmp_v_cons_name == v.cons_name
  && tmp_v_cons_formals == v.cons_formals && tmp_v_cons_throws == v.cons_throws
  && tmp_v_cons_body == v.cons_body) then v
  else { cons_modifiers = tmp_v_cons_modifiers;  cons_name = tmp_v_cons_name; 
        cons_formals = tmp_v_cons_formals;  cons_throws = tmp_v_cons_throws; 
        cons_body = tmp_v_cons_body; }
and visit_method_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_method_decl visit_children_method_decl v
and visit_children_method_decl (vis : jabsVisitor) v =
  let tmp_v_md_modifiers = List.map (fun v -> visit_modifier vis v) v.md_modifiers in
  let tmp_v_md_type = visit_type_or_void vis v.md_type in
  let tmp_v_md_name = visit_ident vis v.md_name in
  let tmp_v_md_formals = List.map (fun v -> visit_formal vis v) v.md_formals in
  let tmp_v_md_throws = List.map (fun v -> visit_qualified_ident vis v) v.md_throws in
  let tmp_v_md_body = match v.md_body with None -> v.md_body
                       | Some(x) -> (let tmp_x = visit_block vis x in
                      if (tmp_x == x) then v.md_body else (Some((tmp_x)))) in
  if (tmp_v_md_modifiers == v.md_modifiers && tmp_v_md_type == v.md_type &&
  tmp_v_md_name == v.md_name && tmp_v_md_formals == v.md_formals &&
  tmp_v_md_throws == v.md_throws && tmp_v_md_body == v.md_body) then v
  else { md_modifiers = tmp_v_md_modifiers;  md_type = tmp_v_md_type; 
        md_name = tmp_v_md_name;  md_formals = tmp_v_md_formals; 
        md_throws = tmp_v_md_throws;  md_body = tmp_v_md_body; }
and visit_formal (vis : jabsVisitor) v =
  doVisit vis vis#v_formal visit_children_formal v
and visit_children_formal (vis : jabsVisitor) v =
  let tmp_v_f_final = v.f_final (* no bool visitor *) in
  let tmp_v_f_type = visit_typ vis v.f_type in
  let tmp_v_f_name = visit_ident vis v.f_name in
  let tmp_v_f_brackets = v.f_brackets (* no int visitor *) in
  if (tmp_v_f_final == v.f_final && tmp_v_f_type == v.f_type &&
  tmp_v_f_name == v.f_name && tmp_v_f_brackets == v.f_brackets) then v
  else { f_final = tmp_v_f_final;  f_type = tmp_v_f_type; 
        f_name = tmp_v_f_name;  f_brackets = tmp_v_f_brackets; }
and visit_block (vis : jabsVisitor) v =
  doVisit vis vis#v_block visit_children_block v
and visit_children_block (vis : jabsVisitor) v =
  List.map (fun v -> visit_block_statement vis v) v
and visit_block_statement (vis : jabsVisitor) v =
  doVisit vis vis#v_block_statement visit_children_block_statement v
and visit_children_block_statement (vis : jabsVisitor) v =
  begin match v with
    LocalVariableDecl a169 -> let tmp_a169 = visit_local_variable_decl vis a169 in
                              if (tmp_a169 == a169) then v
                              else (LocalVariableDecl(tmp_a169))
  | BlockClassDecl a170 -> let tmp_a170 = visit_class_decl vis a170 in
                           if (tmp_a170 == a170) then v
                           else (BlockClassDecl(tmp_a170))
  | Statement a171 -> let tmp_a171 = visit_stmtn vis a171 in
                      if (tmp_a171 == a171) then v else (Statement(tmp_a171))
  end
and visit_local_variable_decl (vis : jabsVisitor) v =
  doVisit vis vis#v_local_variable_decl visit_children_local_variable_decl v
and visit_children_local_variable_decl (vis : jabsVisitor) v =
  let tmp_v_lv_final = v.lv_final (* no bool visitor *) in
  let tmp_v_lv_type = visit_typ vis v.lv_type in
  let tmp_v_lv_vd = List.map (fun v -> visit_variable_declarator vis v) v.lv_vd in
  if (tmp_v_lv_final == v.lv_final && tmp_v_lv_type == v.lv_type &&
  tmp_v_lv_vd == v.lv_vd) then v
  else { lv_final = tmp_v_lv_final;  lv_type = tmp_v_lv_type; 
        lv_vd = tmp_v_lv_vd; }
and visit_stmtn (vis : jabsVisitor) v =
  doVisit vis vis#v_stmtn visit_children_stmtn v
and visit_children_stmtn (vis : jabsVisitor) v =
  { v with node = let v = v.node in visit_stmt vis v; }
and visit_stmt (vis : jabsVisitor) v =
  doVisit vis vis#v_stmt visit_children_stmt v
and visit_children_stmt (vis : jabsVisitor) v =
  begin match v with
    Labeled(a172,a173) -> let tmp_a172 = visit_ident vis a172 in
                          let tmp_a173 = visit_stmtn vis a173 in
                          if (tmp_a172 == a172 && tmp_a173 == a173) then v
                          else (Labeled(tmp_a172 , tmp_a173))
  | IfThen(a174,a175) -> let tmp_a174 = visit_expn vis a174 in
                         let tmp_a175 = visit_stmtn vis a175 in
                         if (tmp_a174 == a174 && tmp_a175 == a175) then v
                         else (IfThen(tmp_a174 , tmp_a175))
  | IfThenElse(a176,a177,a178) -> let tmp_a176 = visit_expn vis a176 in
                                  let tmp_a177 = visit_stmtn vis a177 in
                                  let tmp_a178 = visit_stmtn vis a178 in
                                  if (tmp_a176 == a176 && tmp_a177 == a177 &&
                                  tmp_a178 == a178) then v
                                  else (IfThenElse(tmp_a176 , tmp_a177 ,
                                  tmp_a178))
  | ExpStmt a179 -> let tmp_a179 = visit_exp_stmt vis a179 in
                    if (tmp_a179 == a179) then v else (ExpStmt(tmp_a179))
  | Switch(a180,a181) -> let tmp_a180 = visit_expn vis a180 in
                         let tmp_a181 = visit_switch_block vis a181 in
                         if (tmp_a180 == a180 && tmp_a181 == a181) then v
                         else (Switch(tmp_a180 , tmp_a181))
  | While(a182,a183) -> let tmp_a182 = visit_expn vis a182 in
                        let tmp_a183 = visit_stmtn vis a183 in
                        if (tmp_a182 == a182 && tmp_a183 == a183) then v
                        else (While(tmp_a182 , tmp_a183))
  | Do(a184,a185) -> let tmp_a184 = visit_stmtn vis a184 in
                     let tmp_a185 = visit_expn vis a185 in
                     if (tmp_a184 == a184 && tmp_a185 == a185) then v
                     else (Do(tmp_a184 , tmp_a185))
  | For a186 -> let tmp_a186 = visit_for_stmt vis a186 in
                if (tmp_a186 == a186) then v else (For(tmp_a186))
  | BlockStmt a187 -> let tmp_a187 = visit_block vis a187 in
                      if (tmp_a187 == a187) then v else (BlockStmt(tmp_a187))
  | EmptyStmt -> v
  | Break a188 -> let tmp_a188 = match a188 with None -> a188
                                  | Some(x) -> (let tmp_x = visit_ident vis x in
                                 if (tmp_x == x) then a188 else (Some((tmp_x)))) in
                  if (tmp_a188 == a188) then v else (Break(tmp_a188))
  | Continue a189 -> let tmp_a189 = match a189 with None -> a189
                                     | Some(x) -> (let tmp_x = visit_ident vis x in
                                    if (tmp_x == x) then a189
                                    else (Some((tmp_x)))) in
                     if (tmp_a189 == a189) then v else (Continue(tmp_a189))
  | Return a190 -> let tmp_a190 = match a190 with None -> a190
                                   | Some(x) -> (let tmp_x = visit_expn vis x in
                                  if (tmp_x == x) then a190
                                  else (Some((tmp_x)))) in
                   if (tmp_a190 == a190) then v else (Return(tmp_a190))
  | Throw a191 -> let tmp_a191 = visit_expn vis a191 in
                  if (tmp_a191 == a191) then v else (Throw(tmp_a191))
  | SynchronizedExp(a192,a193) -> let tmp_a192 = visit_expn vis a192 in
                                  let tmp_a193 = visit_block vis a193 in
                                  if (tmp_a192 == a192 && tmp_a193 == a193)
                                  then v else (SynchronizedExp(tmp_a192 ,
                                         tmp_a193))
  | Try a194 -> let tmp_a194 = visit_try_stmt vis a194 in
                if (tmp_a194 == a194) then v else (Try(tmp_a194))
  end
and visit_try_stmt (vis : jabsVisitor) v =
  doVisit vis vis#v_try_stmt visit_children_try_stmt v
and visit_children_try_stmt (vis : jabsVisitor) v =
  let tmp_v_try_block = visit_block vis v.try_block in
  let tmp_v_try_catch = visit_catches vis v.try_catch in
  let tmp_v_try_finally = match v.try_finally with None -> v.try_finally
                           | Some(x) -> (let tmp_x = visit_block vis x in
                          if (tmp_x == x) then v.try_finally
                          else (Some((tmp_x)))) in
  if (tmp_v_try_block == v.try_block && tmp_v_try_catch == v.try_catch &&
  tmp_v_try_finally == v.try_finally) then v
  else { try_block = tmp_v_try_block;  try_catch = tmp_v_try_catch; 
        try_finally = tmp_v_try_finally; }
and visit_catches (vis : jabsVisitor) v =
  doVisit vis vis#v_catches visit_children_catches v
and visit_children_catches (vis : jabsVisitor) v =
  List.map (fun v -> let (a195, a196) = v in
                     let tmp_a195 = visit_formal vis a195 in
                     let tmp_a196 = visit_block vis a196 in
                     if (tmp_a195 == a195 && tmp_a196 == a196) then v
                     else (tmp_a195 , tmp_a196)) v
and visit_for_stmt (vis : jabsVisitor) v =
  doVisit vis vis#v_for_stmt visit_children_for_stmt v
and visit_children_for_stmt (vis : jabsVisitor) v =
  let tmp_v_for_init = match v.for_init with None -> v.for_init
                        | Some(x) -> (let tmp_x = visit_for_init vis x in
                       if (tmp_x == x) then v.for_init else (Some((tmp_x)))) in
  let tmp_v_for_guard = match v.for_guard with None -> v.for_guard
                         | Some(x) -> (let tmp_x = visit_expn vis x in
                        if (tmp_x == x) then v.for_guard else (Some((tmp_x)))) in
  let tmp_v_for_update = List.map (fun v -> visit_esn vis v) v.for_update in
  let tmp_v_for_body = visit_stmtn vis v.for_body in
  if (tmp_v_for_init == v.for_init && tmp_v_for_guard == v.for_guard &&
  tmp_v_for_update == v.for_update && tmp_v_for_body == v.for_body) then v
  else { for_init = tmp_v_for_init;  for_guard = tmp_v_for_guard; 
        for_update = tmp_v_for_update;  for_body = tmp_v_for_body; }
and visit_for_init (vis : jabsVisitor) v =
  doVisit vis vis#v_for_init visit_children_for_init v
and visit_children_for_init (vis : jabsVisitor) v =
  begin match v with
    ForInitStmtExp a197 -> let tmp_a197 = List.map (fun v -> visit_esn vis v) a197 in
                           if (tmp_a197 == a197) then v
                           else (ForInitStmtExp(tmp_a197))
  | ForInitVarDecl a198 -> let tmp_a198 = visit_local_variable_decl vis a198 in
                           if (tmp_a198 == a198) then v
                           else (ForInitVarDecl(tmp_a198))
  end
and visit_switch_block (vis : jabsVisitor) v =
  doVisit vis vis#v_switch_block visit_children_switch_block v
and visit_children_switch_block (vis : jabsVisitor) v =
  let tmp_v_sb_stmt_groups = List.map (fun v -> visit_switch_block_stmt_group vis v) v.sb_stmt_groups in
  if (tmp_v_sb_stmt_groups == v.sb_stmt_groups) then v
  else { sb_stmt_groups = tmp_v_sb_stmt_groups; }
and visit_switch_block_stmt_group (vis : jabsVisitor) v =
  doVisit vis vis#v_switch_block_stmt_group visit_children_switch_block_stmt_group v
and visit_children_switch_block_stmt_group (vis : jabsVisitor) v =
  let tmp_v_sbsg_labels = List.map (fun v -> visit_switch_label vis v) v.sbsg_labels in
  let tmp_v_sbsg_stmts = List.map (fun v -> visit_block_statement vis v) v.sbsg_stmts in
  if (tmp_v_sbsg_labels == v.sbsg_labels && tmp_v_sbsg_stmts == v.sbsg_stmts)
  then v else { sbsg_labels = tmp_v_sbsg_labels; 
               sbsg_stmts = tmp_v_sbsg_stmts; }
and visit_switch_label (vis : jabsVisitor) v =
  doVisit vis vis#v_switch_label visit_children_switch_label v
and visit_children_switch_label (vis : jabsVisitor) v =
  begin match v with
    Default -> v
  | Case a199 -> let tmp_a199 = visit_expn vis a199 in
                 if (tmp_a199 == a199) then v else (Case(tmp_a199))
  end
and visit_esn (vis : jabsVisitor) v =
  doVisit vis vis#v_esn visit_children_esn v
and visit_children_esn (vis : jabsVisitor) v =
  { v with node = let v = v.node in visit_exp_stmt vis v; }
and visit_exp_stmt (vis : jabsVisitor) v =
  doVisit vis vis#v_exp_stmt visit_children_exp_stmt v
and visit_children_exp_stmt (vis : jabsVisitor) v =
  begin match v with
    Assignment(a200,a201,a202) -> let tmp_a200 = visit_aop vis a200 in
                                  let tmp_a201 = visit_expn vis a201 in
                                  let tmp_a202 = visit_expn vis a202 in
                                  if (tmp_a200 == a200 && tmp_a201 == a201 &&
                                  tmp_a202 == a202) then v
                                  else (Assignment(tmp_a200 , tmp_a201 ,
                                  tmp_a202))
  | PreOp(a203,a204) -> let tmp_a203 = visit_preop vis a203 in
                        let tmp_a204 = visit_expn vis a204 in
                        if (tmp_a203 == a203 && tmp_a204 == a204) then v
                        else (PreOp(tmp_a203 , tmp_a204))
  | PostOp(a205,a206) -> let tmp_a205 = visit_expn vis a205 in
                         let tmp_a206 = visit_postop vis a206 in
                         if (tmp_a205 == a205 && tmp_a206 == a206) then v
                         else (PostOp(tmp_a205 , tmp_a206))
  | MethodInvoc(a207,a208) -> let tmp_a207 = visit_ident vis a207 in
                              let tmp_a208 = List.map (fun v -> visit_expn vis v) a208 in
                              if (tmp_a207 == a207 && tmp_a208 == a208) then v
                              else (MethodInvoc(tmp_a207 , tmp_a208))
  | MethodInvocExp(a209,a210,a211) -> let tmp_a209 = visit_expn vis a209 in
                                      let tmp_a210 = visit_ident vis a210 in
                                      let tmp_a211 = List.map (fun v -> visit_expn vis v) a211 in
                                      if (tmp_a209 == a209 && tmp_a210 == a210
                                      && tmp_a211 == a211) then v
                                      else (MethodInvocExp(tmp_a209 , tmp_a210 ,
                                      tmp_a211))
  | MethodInvocSuper(a212,a213) -> let tmp_a212 = match a212 with None -> a212
                                                   | Some(x) -> (let tmp_x = visit_ident vis x in
                                                  if (tmp_x == x) then a212
                                                  else (Some((tmp_x)))) in
                                   let tmp_a213 = List.map (fun v -> visit_expn vis v) a213 in
                                   if (tmp_a212 == a212 && tmp_a213 == a213)
                                   then v else (MethodInvocSuper(tmp_a212 ,
                                          tmp_a213))
  | MethodInvocThis a214 -> let tmp_a214 = List.map (fun v -> visit_expn vis v) a214 in
                            if (tmp_a214 == a214) then v
                            else (MethodInvocThis(tmp_a214))
  | MethodInvocClassSuper(a215,a216,a217) -> let tmp_a215 = visit_qualified_ident vis a215 in
                                             let tmp_a216 = visit_ident vis a216 in
                                             let tmp_a217 = List.map (fun v -> visit_expn vis v) a217 in
                                             if (tmp_a215 == a215 &&
                                             tmp_a216 == a216 &&
                                             tmp_a217 == a217) then v
                                             else (MethodInvocClassSuper(tmp_a215
                                             , tmp_a216 , tmp_a217))
  | ClassInstanceCreationType(a218,a219) -> let tmp_a218 = visit_typ vis a218 in
                                            let tmp_a219 = visit_class_instance_creation vis a219 in
                                            if (tmp_a218 == a218 &&
                                            tmp_a219 == a219) then v
                                            else (ClassInstanceCreationType(tmp_a218
                                            , tmp_a219))
  | ClassInstanceCreationExp(a220,a221,a222) -> let tmp_a220 = visit_expn vis a220 in
                                                let tmp_a221 = visit_ident vis a221 in
                                                let tmp_a222 = visit_class_instance_creation vis a222 in
                                                if (tmp_a220 == a220 &&
                                                tmp_a221 == a221 &&
                                                tmp_a222 == a222) then v
                                                else (ClassInstanceCreationExp(tmp_a220
                                                , tmp_a221 , tmp_a222))
  end
and visit_aop (vis : jabsVisitor) v =
  doVisit vis vis#v_aop visit_children_aop v
and visit_children_aop (vis : jabsVisitor) v =
  begin match v with
    Assign -> v
  | PlusAssign -> v
  | MinusAssign -> v
  | StarAssign -> v
  | SlashAssign -> v
  | AndAssign -> v
  | OrAssign -> v
  | XorAssign -> v
  | RemAssign -> v
  | LShiftAssign -> v
  | RSignedShiftAssign -> v
  | RUnsignedShiftAssign -> v
  end
and visit_preop (vis : jabsVisitor) v =
  doVisit vis vis#v_preop visit_children_preop v
and visit_children_preop (vis : jabsVisitor) v =
  begin match v with
    PreInc -> v
  | PreDec -> v
  | Bang -> v
  | Tilde -> v
  | PrePlus -> v
  | PreMinus -> v
  end
and visit_postop (vis : jabsVisitor) v =
  doVisit vis vis#v_postop visit_children_postop v
and visit_children_postop (vis : jabsVisitor) v =
  begin match v with
    PostInc -> v
  | PostDec -> v
  end
and visit_class_instance_creation (vis : jabsVisitor) v =
  doVisit vis vis#v_class_instance_creation visit_children_class_instance_creation v
and visit_children_class_instance_creation (vis : jabsVisitor) v =
  let tmp_v_cic_args = List.map (fun v -> visit_expn vis v) v.cic_args in
  let tmp_v_cic_body = match v.cic_body with None -> v.cic_body
                        | Some(x) -> (let tmp_x = visit_class_body vis x in
                       if (tmp_x == x) then v.cic_body else (Some((tmp_x)))) in
  if (tmp_v_cic_args == v.cic_args && tmp_v_cic_body == v.cic_body) then v
  else { cic_args = tmp_v_cic_args;  cic_body = tmp_v_cic_body; }
and visit_expn (vis : jabsVisitor) v =
  doVisit vis vis#v_expn visit_children_expn v
and visit_children_expn (vis : jabsVisitor) v =
  { v with node = let v = v.node in visit_exp vis v; }
and visit_exp (vis : jabsVisitor) v =
  doVisit vis vis#v_exp visit_children_exp v
and visit_children_exp (vis : jabsVisitor) v =
  begin match v with
    Literal a223 -> let tmp_a223 = visit_literal vis a223 in
                    if (tmp_a223 == a223) then v else (Literal(tmp_a223))
  | TypeDotClass a224 -> let tmp_a224 = visit_typ vis a224 in
                         if (tmp_a224 == a224) then v
                         else (TypeDotClass(tmp_a224))
  | VoidDotClass -> v
  | This -> v
  | ClassNameDotThis a225 -> let tmp_a225 = visit_qualified_ident vis a225 in
                             if (tmp_a225 == a225) then v
                             else (ClassNameDotThis(tmp_a225))
  | ES a226 -> let tmp_a226 = visit_exp_stmt vis a226 in
               if (tmp_a226 == a226) then v else (ES(tmp_a226))
  | ArrayCreation a227 -> let tmp_a227 = visit_array_creation vis a227 in
                          if (tmp_a227 == a227) then v
                          else (ArrayCreation(tmp_a227))
  | ArrayCreationInit a228 -> let tmp_a228 = visit_array_creation_init vis a228 in
                              if (tmp_a228 == a228) then v
                              else (ArrayCreationInit(tmp_a228))
  | Field(a229,a230) -> let tmp_a229 = visit_expn vis a229 in
                        let tmp_a230 = visit_ident vis a230 in
                        if (tmp_a229 == a229 && tmp_a230 == a230) then v
                        else (Field(tmp_a229 , tmp_a230))
  | SuperField a231 -> let tmp_a231 = visit_ident vis a231 in
                       if (tmp_a231 == a231) then v else (SuperField(tmp_a231))
  | ClassSuperField(a232,a233) -> let tmp_a232 = visit_qualified_ident vis a232 in
                                  let tmp_a233 = visit_ident vis a233 in
                                  if (tmp_a232 == a232 && tmp_a233 == a233)
                                  then v else (ClassSuperField(tmp_a232 ,
                                         tmp_a233))
  | ArrayAccess(a234,a235) -> let tmp_a234 = visit_expn vis a234 in
                              let tmp_a235 = visit_expn vis a235 in
                              if (tmp_a234 == a234 && tmp_a235 == a235) then v
                              else (ArrayAccess(tmp_a234 , tmp_a235))
  | Cast(a236,a237) -> let tmp_a236 = visit_typ vis a236 in
                       let tmp_a237 = visit_expn vis a237 in
                       if (tmp_a236 == a236 && tmp_a237 == a237) then v
                       else (Cast(tmp_a236 , tmp_a237))
  | BinOp(a238,a239,a240) -> let tmp_a238 = visit_bop vis a238 in
                             let tmp_a239 = visit_expn vis a239 in
                             let tmp_a240 = visit_expn vis a240 in
                             if (tmp_a238 == a238 && tmp_a239 == a239 &&
                             tmp_a240 == a240) then v
                             else (BinOp(tmp_a238 , tmp_a239 , tmp_a240))
  | InstanceOf(a241,a242) -> let tmp_a241 = visit_expn vis a241 in
                             let tmp_a242 = visit_typ vis a242 in
                             if (tmp_a241 == a241 && tmp_a242 == a242) then v
                             else (InstanceOf(tmp_a241 , tmp_a242))
  | Conditional(a243,a244,a245) -> let tmp_a243 = visit_expn vis a243 in
                                   let tmp_a244 = visit_expn vis a244 in
                                   let tmp_a245 = visit_expn vis a245 in
                                   if (tmp_a243 == a243 && tmp_a244 == a244 &&
                                   tmp_a245 == a245) then v
                                   else (Conditional(tmp_a243 , tmp_a244 ,
                                   tmp_a245))
  | ExpressionName a246 -> let tmp_a246 = visit_qualified_ident vis a246 in
                           if (tmp_a246 == a246) then v
                           else (ExpressionName(tmp_a246))
  end
and visit_bop (vis : jabsVisitor) v =
  doVisit vis vis#v_bop visit_children_bop v
and visit_children_bop (vis : jabsVisitor) v =
  begin match v with
    SC_Or -> v
  | SC_And -> v
  | Bit_Or -> v
  | Bit_Xor -> v
  | Bit_And -> v
  | Eq -> v
  | Ne -> v
  | Gt -> v
  | Lt -> v
  | Ge -> v
  | Le -> v
  | LShift -> v
  | RSignedShift -> v
  | RUnsignedShift -> v
  | Plus -> v
  | Minus -> v
  | Star -> v
  | Slash -> v
  | Rem -> v
  end
and visit_array_creation (vis : jabsVisitor) v =
  doVisit vis vis#v_array_creation visit_children_array_creation v
and visit_children_array_creation (vis : jabsVisitor) v =
  let tmp_v_ac_type = visit_typ vis v.ac_type in
  let tmp_v_ac_dim_expr = List.map (fun v -> visit_expn vis v) v.ac_dim_expr in
  let tmp_v_ac_dims = v.ac_dims (* no int visitor *) in
  if (tmp_v_ac_type == v.ac_type && tmp_v_ac_dim_expr == v.ac_dim_expr &&
  tmp_v_ac_dims == v.ac_dims) then v
  else { ac_type = tmp_v_ac_type;  ac_dim_expr = tmp_v_ac_dim_expr; 
        ac_dims = tmp_v_ac_dims; }
and visit_array_creation_init (vis : jabsVisitor) v =
  doVisit vis vis#v_array_creation_init visit_children_array_creation_init v
and visit_children_array_creation_init (vis : jabsVisitor) v =
  let tmp_v_aci_type = visit_typ vis v.aci_type in
  let tmp_v_aci_dims = v.aci_dims (* no int visitor *) in
  let tmp_v_aci_init = visit_array_initializer vis v.aci_init in
  if (tmp_v_aci_type == v.aci_type && tmp_v_aci_dims == v.aci_dims &&
  tmp_v_aci_init == v.aci_init) then v
  else { aci_type = tmp_v_aci_type;  aci_dims = tmp_v_aci_dims; 
        aci_init = tmp_v_aci_init; }
and visit_literal (vis : jabsVisitor) v =
  doVisit vis vis#v_literal visit_children_literal v
and visit_children_literal (vis : jabsVisitor) v =
  begin match v with
    IntLit a247 -> let tmp_a247 = a247 (* no string visitor *) in
                   if (tmp_a247 == a247) then v else (IntLit(tmp_a247))
  | FloatLit a248 -> let tmp_a248 = a248 (* no string visitor *) in
                     if (tmp_a248 == a248) then v else (FloatLit(tmp_a248))
  | CharLit a249 -> let tmp_a249 = a249 (* no string visitor *) in
                    if (tmp_a249 == a249) then v else (CharLit(tmp_a249))
  | StringLit a250 -> let tmp_a250 = a250 (* no string visitor *) in
                      if (tmp_a250 == a250) then v else (StringLit(tmp_a250))
  | True -> v
  | False -> v
  | Null -> v
  end
